        ;-----------------------------------------------------------------------
        ; RASTER4 by Marcus Geelnard, 2014
        ; This is a 256-byte demo for C=64.
        ;-----------------------------------------------------------------------
        ; This is free and unencumbered software released into the public
        ; domain.
        ;
        ; Anyone is free to copy, modify, publish, use, compile, sell, or
        ; distribute this software, either in source code form or as a compiled
        ; binary, for any purpose, commercial or non-commercial, and by any
        ; means.
        ;
        ; In jurisdictions that recognize copyright laws, the author or authors
        ; of this software dedicate any and all copyright interest in the
        ; software to the public domain. We make this dedication for the benefit
        ; of the public at large and to the detriment of our heirs and
        ; successors. We intend this dedication to be an overt act of
        ; relinquishment in perpetuity of all present and future rights to this
        ; software under copyright law.
        ;
        ; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
        ; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        ; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
        ; IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
        ; OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
        ; ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
        ; OTHER DEALINGS IN THE SOFTWARE.
        ;
        ; For more information, please refer to <http://unlicense.org/>
        ;-----------------------------------------------------------------------
        ; Compile with Aart Bik's 6510 Cross-Assembler:
        ;  http://www.aartbik.com/MISC/c64.html
        ;
        ; Quick cross-dev compile & run-in-VICE-emulator one-liner under Linux:
        ;  lin2c64 rasters4.s && x64 rasters4.t64
        ;-----------------------------------------------------------------------

cos_pos     .equ  $2
bar_cnt     .equ  $3
bar_delta   .equ  $4
cur_note    .equ  $6

start_line  .equ  75
colors      .equ  $3000

        .org  $0801

; "2014 SYS2062" - in BASIC program space
        .word $0801
        .word 2014
        .byte $9e $32 $30 $36 $32
        .word $0000
        .word $0000

; Address here is 2062
start_2062

        ;-----------------------------------------------------------------------
        ; Program init
        ;-----------------------------------------------------------------------

        sei

        ; Turn off graphics DMA (borders all over!)
        ldx #0
        stx $d011

        ; Clear sound
        txa
        tay         ; We need a zero in Y later
clr_snd
        sta $d400,x
        inx
        bne clr_snd

        ; Sound configuration
        dex         ; x = #$ff
        stx $d40d   ; voice 2 sustain volume = 15
        stx $d417   ; activate filter (resonance = 15, filter all channels)

        lda #$2e
        sta $d405   ; voice 1 attack and decay lengths
        sta $d418   ; volume and filter setting (bandpass, vol = 14)

        ; Build remaining part of cosine table (mirror)
        ; NOTE: Y must be zero (see tay above)
        ldx #63
cos_loop
        lda cos_table,x
        sta cos_table_mirror,y
        iny
        dex
        bne cos_loop

        ; Clear raster bar delta (8.8 fixed point)
        stx bar_delta
        stx bar_delta+1

start_sound
        ; Get current note frequency
        lda bar_delta
        and #7
        tay
        lda notes,y
        sta cur_note

        ; Set frequency
        sta $d401   ; voice 1 frequency (hi bits)
        sta $d408   ; voice 2 frequency (hi bits)
        sta $d407   ; voice 2 frequency (lo bits - this does the detune)
        sta $d416   ; filter frequency

        ; Start sound
        lda #$21
        sta $d404   ; voice 1: start saw wave
        sta $d40b   ; voice 2: start saw wave

        ; NOTE: X must be 0 when entering the main_loop!
main_loop

        ;-----------------------------------------------------------------------
        ; Generate raster bar data
        ;-----------------------------------------------------------------------

        ; Clear colors (we clear 256 lines here, just to save a couple of bytes
        ; of code)
        lda cur_note
clear_colors
        sta colors,x
        inx
        bne clear_colors

        ; Select the next cos-table position
        inc cos_pos
        lda cos_pos

        ; Do a few raster bars (the CPU limit is about 16 bars)
        ldx #12
        stx bar_cnt
bars_loop
        ; Set raster bar position
        and #127
        tay
        lda cos_table,y
        sta draw_bar+4    ; Self-modifying code FTW!

        ; Draw one bar
        ldx #21
draw_bar
        lda bar-1,x
        sta colors,x
        dex
        bne draw_bar

        ; Increment raster cos-table position for the next bar
        tya
        clc
        adc bar_delta

        dec bar_cnt
        bne bars_loop


        ;-----------------------------------------------------------------------
        ; Display raster bars
        ;-----------------------------------------------------------------------

        ; Main raster loop
        ldx #start_line
next_line
        lda colors-start_line,x

wait_line
        cpx $d012
        bne wait_line

        sta $d020

        inx
        bne next_line

        ; Important: At this point, X is zero (see main_loop).

        ; Increment bar delta (distance between bars)
        inc bar_delta+1
        inc bar_delta+1
        bne main_loop

        inc bar_delta

        stx $d404   ; voice 1: stop (will be retriggered in start_sound)

        jmp start_sound


        ;-----------------------------------------------------------------------
        ; Pre-defined data
        ;-----------------------------------------------------------------------

        ; 21-entry raster bar definition
bar     .byte 9,2,9,2,8,2,8,10,8,10
        .byte 7
        .byte 10,8,10,8,2,8,2,9,2,9

        ; This is the "melody", consisting of eight equal length notes
notes   .byte 4,5,3,4,6,5,3,4

        ; 64-entry half cosine table
cos_table
        .byte 155,155,155,154,154,153,152,150,149,148,146,144,142,140,137,135
        .byte 132,130,127,124,121,117,114,111,107,104,100,96,93,89,85,81
        .byte 77,74,70,66,62,59,55,51,48,44,41,38,34,31,28,25,23
        .byte 20,18,15,13,11,9,7,6,5,3,2,1,1,0,0,0
cos_table_mirror

